<?php

namespace Drupal\entity_access_by_role\Plugin\Field\FieldType;

use Drupal\Component\Utility\Unicode;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\options\Plugin\Field\FieldType\ListItemBase;

/**
 * Plugin implementation of the 'role_access' field type.
 *
 * @FieldType(
 *   id = "role_access",
 *   label = @Translation("Role Access"),
 *   module = "entity_access_by_role",
 *   description = @Translation("Access Control Field"),
 *   default_widget = "role_access_widget",
 * )
 */
class RoleAccessItem extends ListItemBase {

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {

    $properties['value'] = DataDefinition::create('string')
      ->setLabel(t('Text value'))
      ->addConstraint('Length', ['max' => 255])
      ->setRequired(TRUE);

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {

    return [
      'columns' => [
        'value' => [
          'type'   => 'varchar',
          'length' => 255,
        ],
      ],
      'indexes' => [
        'value' => ['value'],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings() {
    return [
      'allowed_values' => [],
      'allowed_values_function' => 'entity_access_by_role_roles_without_bypass_access',
    ] + parent::defaultStorageSettings();
  }

  /**
   * {@inheritdoc}
   */
  protected function allowedValuesDescription() {

    $description = '<p>' . t('The values of this field are populated by the roles available on your site and will be updated automatically.</p>');
    return $description;
  }

  /**
   * {@inheritdoc}
   */
  protected static function validateAllowedValue($option) {

    if (Unicode::strlen($option) > 255) {
      return t('Allowed values list: each key must be a string at most 255 characters long.');
    }
  }

  /**
   * {@inheritdoc}
   */
  protected static function castAllowedValue($value) {

    return (string) $value;
  }

}
